from config import *

wave_speed = 0.3
class Ocean:
    def __init__(self):
        screen.fill((52, 189, 235))
        self.animations = []
        self.wave_gen_cycle = Cycle(2, 20)

    def draw(self, offset=(0, 0)):
        screen.fill((52, 189, 235))

        for i in range(len(self.animations)-1, -1, -1):
            self.animations[i].draw(offset)
            if self.animations[i].is_dead():
                if self.animations[i].name == "fish_jump":
                    self.animations.append(RippleAnimation(
                        [self.animations[i].pos[0] - 17, self.animations[i].pos[1] + 30]))
                del self.animations[i]

    def update(self, offset=(0, 0)):
        if random.randint(0, 20) == 0:
            self.animations.append(WaveAnimation([random.randint(0, screen_width) - offset[0],
                                                 random.randint(0, screen_height) - offset[1]]))
        if random.randint(0, 80) == 0:
            fish_pos = [random.randint(0, screen_width) - offset[0], random.randint(0, screen_height) - offset[1]]
            self.animations.append(Animation(
                fish_pos,
                5, 3, 50, 80, "fish_jump"))
            self.animations.append(RippleAnimation([fish_pos[0] + 13, fish_pos[1] + 30]))


class Animation:
    def __init__(self, pos, length, speed, width, height, txt_name):
        self.draw_cycle = Cycle(length, speed)
        self.pos = pos
        self.name = txt_name
        self.height = height
        self.width = width

    def is_dead(self):
        return self.draw_cycle.one

    def draw(self, offset=(0, 0)):
        current = self.draw_cycle.get()
        screen.blit(texture_lib[self.name], (self.pos[0] + offset[0], self.pos[1] + offset[1]),
                    pygame.Rect(0, current*self.height, self.width, self.height))


class RippleAnimation(Animation):
    def __init__(self, pos):
        Animation.__init__(self, pos, 4, 3, 50, 50, "ripple")


class WaveAnimation:
    def __init__(self, pos):
        self.draw_cycle = Cycle(11, 3)
        self.pos = pos
        self.name = "little_wave"

    def is_dead(self):
        return self.draw_cycle.one

    def draw(self, offset=(0, 0)):
        current = self.draw_cycle.get()
        if current < 6:
            screen.blit(texture_lib["little_wave"], (self.pos[0] + offset[0], self.pos[1] + offset[1]),
                        pygame.Rect(0, current*30, 60, 30))
        else:
            screen.blit(texture_lib["little_wave"], (self.pos[0] + offset[0], self.pos[1] + offset[1]),
                        pygame.Rect(0, ((11 - current) * 30), 60, 30))
        self.pos[0] += wave_speed


class WalkDust(Animation):
    def __init__(self, pos, left):
        Animation.__init__(self, pos, 7, 1, 50, 50, "walk_dust")
        if left:
            self.name = "walk_dust_left"
            self.pos = (pos[0], pos[1] + 35)
        else:
            self.name = "walk_dust"
            self.pos = (pos[0] + 30, pos[1] + 35)

