import pygame, sys, os
from cycle import *

pygame.init()

screen_width, screen_height = 600, 600
screen = pygame.display.set_mode((screen_width, screen_height))

clock = pygame.time.Clock()

textures = os.listdir("textures")



def load(n):
    return pygame.image.load(os.path.join("textures", n + ".png")).convert_alpha()


texture_names = [name[:-4] for name in textures]

texture_lib = {}

item_font = pygame.font.Font("Aller_Rg.ttf", 15)
mission_font = pygame.font.Font("Aller_Rg.ttf", 17)
hand_pos = (((16, 40),
             (17, 45),
             (19, 49),
             (17, 45)),
            ((59, 40),
             (58, 45),
             (56, 49),
             (58, 45)),
            ((27, 43),
             (36, 44),
             (47, 50),
             (53, 49),
             (54, 36),
             (55, 42),
             (47, 49),
             (39, 49)),
            ((48, 43),
             (39, 44),
             (28, 50),
             (22, 49),
             (21, 36),
             (20, 42),
             (28, 49),
             (36, 49)))
def add_left(names):
    for name in names:
        texture_lib[name+"_left"] = pygame.transform.flip(texture_lib[name], True, False)

for name in texture_names:
    texture_lib[name] = load(name)

add_left(["player_stand", "player_walk", "walk_dust"])

env_vars = {"mouse_x": 0, "mouse_y": 0, "delta_time": 0, "delta_x": 0, "delta_y": 0,
            "mouse_button_down": False, "mouse_button_changed": False, "keys": pygame.key.get_pressed()}
player_size = 75
tile_size = 50
default_center = (screen_width/2 - player_size/2, screen_height/2 - player_size/2)

plantable = ("item_sapling", "item_melon_seed")
placable = ("item_soil", "item_workbench", "item_furnace", "item_bucket")

handcrafts = {("item_workbench", 1): (("item_wood", 3), ("item_stone", 1)),
              ("item_melon_seed", 5): (("item_melon", 1),)}
benchcrafts = {("item_bucket", 1): (("item_wood", 6),),
               ("item_string", 1): (("item_grass", 1),),
               ("item_axe", 1): (("item_stone", 2), ("item_string", 2), ("item_wood", 3)),
               ("item_pickaxe", 1): (("item_stone", 3), ("item_string", 2), ("item_wood", 3)),
               ("item_furnace", 1): (("item_stone", 10),),
               ("item_shovel", 1): (("item_iron", 4), ("item_string", 4), ("item_wood", 4),),
               ("item_flare", 1): (("item_sulfur", 4), ("item_string", 4))}


class AudioPlayer:
    def __init__(self, id_in, type_in, volume=0.5):
        self.id = id_in
        self.music = pygame.mixer.Sound(os.path.join('audio', id_in + '.ogg'))
        self.music.set_volume(volume)
        if type_in == "bg":
            self.channel = pygame.mixer.Channel(1)
        if type_in == "se":
            self.channel = pygame.mixer.Channel(2)
        if type_in == "sn":
            self.channel = pygame.mixer.Channel(3)

    def play_non_stop(self):
        self.channel.play(self.music, -1)

    def play_once(self):
        self.channel.play(self.music, 0)

    def stop(self):
        self.channel.stop()


bgs = (AudioPlayer("menu", "bg"),
       AudioPlayer("tomorrow", "bg", 0.2),)

ses = (AudioPlayer("sebuild", "se"),
       AudioPlayer("seeat", "se"),
       AudioPlayer("sestep", "sn"),
       AudioPlayer("semissioncomplete", "se"),
       AudioPlayer("sedig", "se"),)
