from platforms import *
from island import *
from missions import *
from player import *


class Mission:
    def __init__(self):
        self.stage = 0
        self.current = 0
        self.finished = False

    def draw(self, map_on=False):
        if self.stage < len(missions):
            if map_on:
                hints = missions[self.stage][3].split("&")
                screen.blit(mission_font.render(missions[self.stage][0] +
                                                "(" + str(self.current) + "/" + str(missions[self.stage][2]) + ")",
                                                False, (30, 30, 30)), (280, 10))
                screen.blit(texture_lib[missions[self.stage][1]], (250, 5))
                for index, hint in enumerate(hints):
                    screen.blit(mission_font.render(hint,
                                                    False, (30, 30, 30), (255, 255, 255)), (270, 35 + index * 25))
            else:
                screen.blit(mission_font.render("Objective: " + missions[self.stage][0] +
                                                "(" + str(self.current) + "/" + str(missions[self.stage][2]) + ")",
                                                False, (30, 30, 30)), (20, 20))
                screen.blit(texture_lib[missions[self.stage][1]], (20, 40 + map_on * 240))

    def update(self, inventory):
        if self.stage < len(missions):
            self.current = 0
            for slot in inventory.pack:
                if slot.item == missions[self.stage][1]:
                    self.current = slot.count
                    if self.current >= missions[self.stage][2]:
                        self.stage += 1
                        ses[3].play_once()
                    return
        else:
            self.finished = True


class Game:
    def __init__(self):
        self.platform = Platform(self)
        self.ocean = Ocean()
        self.boat_speed = 3
        self.inventory = Inventory()
        self.islands = (Island1(),
                        Island2(),
                        Island3(),
                        Island4(),
                        Island5(),
                        Island6(),
                        Island7(),
                        Island8(),
                        )
        self.player = Player(self.platform, self.islands, self.inventory)

        self.offset = (0, 0)
        self.crafting = Crafting(self.inventory, handcrafts)

        self.stat = "nothing"

        self.platform_speed = (0, 0)

        self.map_on = False
        self.map_cycle = Cycle(2, 10)
        self.hint_cycle = Cycle(2, 20)

        self.mission = Mission()
        self.hunger = 200
        self.thirst = 200

    def draw_hunger_thirst(self):
        screen.blit(texture_lib["hunger"], (310, 570), pygame.Rect(0, 0, 220, 20))
        screen.blit(texture_lib["thirst"], (70, 570), pygame.Rect(0, 0, 220, 20))
        screen.blit(texture_lib["hunger"], (310, 570), pygame.Rect(0, 20, 20+int(self.hunger), 20))
        screen.blit(texture_lib["thirst"], (70, 570), pygame.Rect(0, 20, 20+int(self.thirst), 20))

    def draw_minimap(self):
        screen.blit(texture_lib["map"], (20, 20))
        if self.map_cycle.get() == 0:
            pos = [self.player.pos[0]//25 + 29 + 107, self.player.pos[1]//25 + 29 + 105]
            if pos[0] < 40:
                pos[0] = 40
            elif pos[0] > 238:
                pos[0] = 238
            if pos[1] < 40:
                pos[1] = 40
            elif pos[1] > 238:
                pos[1] = 238
            pygame.draw.rect(screen, (235, 61, 52), pygame.Rect(pos[0], pos[1],
                                                                4, 4))
        if self.hint_cycle.get() == 0:
            if self.mission.stage == 2:
                screen.blit(texture_lib["hint"], (17 + 137, 17 + 93))
            elif self.mission.stage == 5:
                screen.blit(texture_lib["hint"], (17 + 164, 17 + 142))
            elif self.mission.stage == 6:
                screen.blit(texture_lib["hint"], (17 + 164, 17 + 142))
            elif self.mission.stage == 9:
                screen.blit(texture_lib["hint"], (17 + 119, 17 + 182))
            elif self.mission.stage == 10:
                screen.blit(texture_lib["hint"], (17 + 68, 17 + 130))
            elif self.mission.stage == 12:
                screen.blit(texture_lib["hint"], (17 + 46, 17 + 180))
            elif self.mission.stage == 13:
                screen.blit(texture_lib["hint"], (17 + 48, 17 + 28))
            elif self.mission.stage == 18:
                screen.blit(texture_lib["hint"], (17 + 100, 17 + 39))

    def move_platform(self):
        if type(self.player.is_on) is Platform:
            self.platform.move(self.platform_speed[0], self.platform_speed[1])
            if is_platform_collide_islands(self.islands, self.platform):
                self.platform.move(-self.platform_speed[0], -self.platform_speed[1])
            else:
                self.player.pos[0] += self.platform_speed[0]
                self.player.pos[1] += self.platform_speed[1]

    def draw(self):
        self.ocean.draw(self.offset)
        for island in self.islands:
            island.draw(self.offset)
        self.platform.draw(self.offset)
        for island in self.islands:
            island.render_back(self.player.pos[1], self.offset)
        self.platform.draw_items_back(self.player.pos[1] + 71, self.offset)
        self.player.draw_shadow()
        self.player.draw()
        for island in self.islands:
            island.render_fore(self.player.pos[1], self.offset)
        self.platform.draw_items_fore(self.player.pos[1] + 71, self.offset)

        if self.stat == "building":
            self.platform.build(self.offset)
        elif self.stat == "crafting":
            self.crafting.draw()
        elif self.stat != "backpack":
            self.inventory.draw_inventory()
            result = False
            if 200 < env_vars["mouse_x"] < 400 and 200 < env_vars["mouse_y"] < 400:
                result = self.platform.attempt_harvest(self.offset)
            if result:
                for item in result:
                    self.inventory.put_item(item[0], item[1])
            else:
                if self.player.is_item_plantable():
                    if self.platform.attempt_put(self.player.get_holding_item().item, self.offset):
                        self.player.get_holding_item().take(1)
                elif self.player.is_item_placable():
                    if self.platform.attempt_place(self.player.get_holding_item().item, self.offset):
                        self.player.get_holding_item().take(1)
                elif self.player.get_holding_item().item == "item_melon":
                    if env_vars["mouse_button_down"] and env_vars["mouse_button_changed"]:
                        self.hunger += 30
                        self.thirst += 5
                        if self.hunger > 200:
                            self.hunger = 200
                        elif self.thirst > 200:
                            self.thirst = 200
                        self.player.crunch = Animation((280, 270), 5, 3, 50, 50, "crunch")
                        self.inventory.take_item("item_melon", 1)
                        ses[1].play_once()
                elif self.player.get_holding_item().item == "item_coconut":
                    if env_vars["mouse_button_down"] and env_vars["mouse_button_changed"]:
                        self.hunger = 200
                        self.thirst = 200
                        self.player.crunch = Animation((280, 270), 5, 3, 50, 50, "crunch")
                        self.inventory.take_item("item_coconut", 1)
                        ses[1].play_once()

        if self.player.is_on in self.islands:
            if 200 < env_vars["mouse_x"] < 400 and 200 < env_vars["mouse_y"] < 400:
                result = self.player.is_on.pick(self.inventory.pack[self.inventory.selected].item, self.offset)
                if result:
                    ses[4].play_once()
                    for item in result:
                        self.inventory.put_item(item[0], item[1])
                        if item[0] == "item_soil":
                            self.inventory.take_item("item_bucket", 1)
        if self.map_on:
            self.draw_minimap()
            self.mission.draw(map_on=True)
        else:
            self.mission.draw()
        if self.stat == "backpack":
            self.inventory.draw_pack()
        self.draw_hunger_thirst()
        if self.stat != "nothing":
            screen.blit(mission_font.render(self.stat, False, (255, 255, 255)), (272, 472))
            screen.blit(mission_font.render(self.stat, False, (0, 0, 0)), (270, 470))

    def update(self):
        self.player.update()
        if self.platform.anchor.direction == 1:
            self.platform_speed = (self.boat_speed, 0)
        elif self.platform.anchor.direction == 3:
            self.platform_speed = (-self.boat_speed, 0)
        elif self.platform.anchor.direction == 4:
            self.platform_speed = (0, -self.boat_speed)
        elif self.platform.anchor.direction == 2:
            self.platform_speed = (0, self.boat_speed)
        elif self.platform.anchor.direction == 0:
            self.platform_speed = (0, 0)
        self.move_platform()
        self.offset = (-self.player.pos[0]+screen_width/2-player_size/2,
                       -self.player.pos[1]+screen_height/2-player_size/2)
        self.ocean.update(self.offset)

        if env_vars["keys"][pygame.K_b]:
            self.stat = "building"
        elif env_vars["keys"][pygame.K_p]:
            self.stat = "backpack"
        elif env_vars["keys"][pygame.K_SPACE]:
            self.stat = "nothing"
        elif env_vars["keys"][pygame.K_c]:
            self.stat = "crafting"
            self.crafting = Crafting(self.inventory, handcrafts)
        if self.stat == "building":
            if self.player.is_on != self.platform:
                self.stat = "nothing"
        if self.stat == "building":
            if not self.inventory.has("item_wood", 1):
                self.stat = "nothing"
        self.mission.update(self.inventory)

        self.hunger -= 0.01
        self.thirst -= 0.003

    def handle_event(self, event):
        if self.stat == "crafting":
            if self.crafting.handle_events(event):
                self.stat = "nothing"
        else:
            self.player.handle_event(event)
            self.inventory.handle_event(event)
        if event.type == pygame.KEYUP:
            if event.key == pygame.K_m:
                self.map_on = not self.map_on
            elif event.key == pygame.K_1:
                self.inventory.selected = 0
            elif event.key == pygame.K_2:
                self.inventory.selected = 1
            elif event.key == pygame.K_3:
                self.inventory.selected = 2
            elif event.key == pygame.K_4:
                self.inventory.selected = 3
            elif event.key == pygame.K_5:
                self.inventory.selected = 4
        elif self.stat == "nothing" and event.type == pygame.MOUSEBUTTONUP:
            if event.button == 1:
                tx, ty = self.platform.hovered_tile((env_vars["mouse_x"] - self.offset[0],
                                                     env_vars["mouse_y"] - self.offset[1]))
                if self.platform.is_on_platform((tx, ty)):
                    if self.platform.area[ty][tx].on_top == "workbench":
                        self.stat = "crafting"
                        self.crafting = Crafting(self.inventory, benchcrafts)
                    elif self.player.get_holding_item().item == "item_iron_ore":
                        if self.platform.area[ty][tx].on_top == "furnace":
                            self.inventory.take_item("item_iron_ore", 1)
                            self.inventory.put_item("item_iron", 1)
