from config import *


class Chunk:
    def __init__(self, base="empty"):
        self.base = base

    def draw(self, pos):
        if self.base != "empty":
            screen.blit(texture_lib[self.base], pos)


class Island:
    def __init__(self):
        self.collide_box = ()
        self.area = []
        self.size = (0, 0)
        self.pos = (0, 0)
        self.items = []

    def gen_items(self, item, count):
        for _ in range(count):
            tx, ty = random.randint(0, self.size[0]-1), random.randint(0, self.size[1]-1)
            while self.area[ty][tx].base == "empty" or self.items[ty][tx] != "empty":
                tx, ty = random.randint(0, self.size[0] - 1), random.randint(0, self.size[1] - 1)
            self.items[ty][tx] = item

    def render_back(self, player_y, offset):
        for y in range(self.size[1]):
            for x in range(self.size[0]):
                if self.pos[1] + y * tile_size + tile_size > player_y + 71:
                    return
                if self.items[y][x] != "empty":
                    if self.items[y][x] == "coconut_tree" or self.items[y][x] == "tall_tree":
                        screen.blit(texture_lib[self.items[y][x]], (self.pos[0] + x * tile_size + offset[0],
                                                                    self.pos[1] + y * tile_size + offset[1] - 100))
                    elif self.items[y][x] == "tall_grass":
                        screen.blit(texture_lib[self.items[y][x]], (self.pos[0] + x * tile_size + offset[0],
                                                                    self.pos[1] + y * tile_size + offset[1] - 50))
                    else:
                        screen.blit(texture_lib[self.items[y][x]], (self.pos[0] + x*tile_size + offset[0],
                                                                    self.pos[1] + y*tile_size + offset[1]))

    def render_fore(self, player_y, offset):
        for y in range(self.size[1]):
            for x in range(self.size[0]):
                if self.pos[1] + y * tile_size + tile_size > player_y + 71:
                    if self.items[y][x] != "empty":
                        if self.items[y][x] == "coconut_tree" or self.items[y][x] == "tall_tree":
                            screen.blit(texture_lib[self.items[y][x]], (self.pos[0] + x * tile_size + offset[0],
                                                                        self.pos[1] + y * tile_size + offset[1] - 100))
                        elif self.items[y][x] == "tall_grass":
                            screen.blit(texture_lib[self.items[y][x]], (self.pos[0] + x * tile_size + offset[0],
                                                                        self.pos[1] + y * tile_size + offset[1] - 50))
                        else:
                            screen.blit(texture_lib[self.items[y][x]], (self.pos[0] + x*tile_size + offset[0],
                                                                        self.pos[1] + y*tile_size + offset[1]))

    def init_items(self):
        self.items = [["empty" for _ in range(self.size[0])] for _ in range(self.size[1])]

    def init_area(self, str_area):
        for y in range(len(str_area)):
            self.area.append([])
            for x in range(len(str_area[y])):
                if str_area[y][x] == 'g':
                    self.area[y].append(Chunk("grass"))
                elif str_area[y][x] == 's':
                    self.area[y].append(Chunk("sand"))
                elif str_area[y][x] == 'r':
                    self.area[y].append(Chunk("stone"))
                else:
                    self.area[y].append(Chunk())

    def init_from_file(self, file_name):
        file = open(os.path.join("maps", file_name + ".txt"), "r")
        info = file.readlines()
        self.init_area(info)
        file.close()

    def is_player_on(self, player):
        for box in self.collide_box:
            n = box.inflate(10, 10)
            if n.collidepoint(player):
                tx, ty = self.get_grid_pos(player)
                if tx >= self.size[0]:
                    tx -= 1
                if ty >= self.size[1]:
                    ty -= 1
                if self.items[ty][tx] != "empty":
                    return False
                return True
        return False

    def collied_rect(self, rect):
        for box in self.collide_box:
            if box.colliderect(rect):
                return True
        return False

    def draw(self, offset):
        for y in range(self.size[1]):
            for x in range(self.size[0]):
                self.area[y][x].draw((offset[0]+self.pos[0]+x*tile_size, offset[1]+self.pos[1]+y*tile_size))

    def get_grid_pos(self, pos, offset=(0, 0)):
        return int((pos[0] - self.pos[0]-offset[0])//50), int((pos[1] - self.pos[1]-offset[1])//50)

    def pick(self, hand, offset):
        tx, ty = self.get_grid_pos((env_vars["mouse_x"], env_vars["mouse_y"]), offset)
        if 0 <= tx < self.size[0] and 0 <= ty < self.size[1]:
            it = self.items[ty][tx]
            picked = env_vars["mouse_button_down"] and env_vars["mouse_button_changed"]
            if hand == "item_pickaxe":
                if it == "iron_ore":
                    pygame.draw.rect(screen, (3, 252, 132), pygame.Rect(self.pos[0] + tx*tile_size + offset[0],
                                                                        self.pos[1] + ty*tile_size + offset[1],
                                                                        tile_size,
                                                                        tile_size), 2)
                    if picked:
                        self.items[ty][tx] = "empty"
                        return ("item_stone", 3), ("item_iron_ore", 1)
                elif it == "sulfur_ore":
                    pygame.draw.rect(screen, (3, 252, 132), pygame.Rect(self.pos[0] + tx * tile_size + offset[0],
                                                                        self.pos[1] + ty * tile_size + offset[1],
                                                                        tile_size,
                                                                        tile_size), 2)
                    if picked:
                        self.items[ty][tx] = "empty"
                        return ("item_stone", 3), ("item_sulfur", 1)
            elif hand == "item_axe":
                if it == "tall_tree":
                    pygame.draw.rect(screen, (3, 252, 132), pygame.Rect(self.pos[0] + tx * tile_size + offset[0],
                                                                        self.pos[1] + ty * tile_size + offset[1],
                                                                        tile_size,
                                                                        tile_size), 2)
                    if picked:
                        self.items[ty][tx] = "empty"
                        return ("item_wood", 6), ("item_sapling", 1)
                elif it == "coconut_tree":
                    pygame.draw.rect(screen, (3, 252, 132), pygame.Rect(self.pos[0] + tx * tile_size + offset[0],
                                                                        self.pos[1] + ty * tile_size + offset[1],
                                                                        tile_size,
                                                                        tile_size), 2)
                    if picked:
                        self.items[ty][tx] = "empty"
                        return ("item_wood", 4), ("item_coconut", 1)
            elif hand == "item_shovel":
                if it == "wreck":
                    pygame.draw.rect(screen, (3, 252, 132), pygame.Rect(self.pos[0] + tx * tile_size + offset[0],
                                                                        self.pos[1] + ty * tile_size + offset[1],
                                                                        tile_size,
                                                                        tile_size), 2)
                    if picked:
                        self.items[ty][tx] = "empty"
                        return ("item_emitter", 1),
            elif hand == "item_bucket":
                if self.area[ty][tx].base == "grass":
                    pygame.draw.rect(screen, (3, 252, 132), pygame.Rect(self.pos[0] + tx * tile_size + offset[0],
                                                                        self.pos[1] + ty * tile_size + offset[1],
                                                                        tile_size,
                                                                        tile_size), 2)
                    if picked:
                        return ("item_soil", 1),
            else:
                if it == "tall_grass":
                    pygame.draw.rect(screen, (3, 252, 132), pygame.Rect(self.pos[0] + tx * tile_size + offset[0],
                                                                        self.pos[1] + ty * tile_size + offset[1],
                                                                        tile_size,
                                                                        tile_size), 2)
                    if picked:
                        self.items[ty][tx] = "empty"
                        return ("item_grass", 1),
                elif it == "rocks":
                    pygame.draw.rect(screen, (3, 252, 132), pygame.Rect(self.pos[0] + tx * tile_size + offset[0],
                                                                        self.pos[1] + ty * tile_size + offset[1],
                                                                        tile_size,
                                                                        tile_size), 2)
                    if picked:
                        self.items[ty][tx] = "empty"
                        return ("item_stone", 1),
        return False


class Island1(Island):
    def __init__(self):
        Island.__init__(self)
        self.init_from_file("map1")
        self.size = (18, 15)
        self.pos = (-2300, -2200)
        self.collide_box = (pygame.Rect(self.pos[0] + 100, self.pos[1],
                                        800, 200),
                            pygame.Rect(self.pos[0], self.pos[1] + 100,
                                        200, 650))
        self.init_items()
        self.gen_items("iron_ore", 12)


class Island2(Island):
    def __init__(self):
        Island.__init__(self)
        self.init_from_file("map2")
        self.size = (10, 5)
        self.pos = (-600, -2000)
        self.collide_box = (pygame.Rect(self.pos[0], self.pos[1],
                                        self.size[0] * tile_size, self.size[1] * tile_size),)
        self.init_items()
        self.gen_items("wreck", 1)


class Island3(Island):
    def __init__(self):
        Island.__init__(self)
        self.init_from_file("map3")
        self.size = (10, 11)
        self.pos = (1150, -1850)
        self.collide_box = (pygame.Rect(self.pos[0], self.pos[1],
                                        350, 350),
                            pygame.Rect(self.pos[0] + 150, self.pos[1] + 150,
                                        350, 400)
                            )
        self.init_items()
        self.gen_items("goat", 10)
        self.gen_items("goat2", 5)


class Island4(Island):
    def __init__(self):
        Island.__init__(self)
        self.init_from_file("map4")
        self.size = (11, 11)
        self.pos = (-1500, 150)
        self.collide_box = (pygame.Rect(self.pos[0], self.pos[1],
                                        550, 200),
                            pygame.Rect(self.pos[0], self.pos[1] + 350,
                                        550, 200),
                            pygame.Rect(self.pos[0], self.pos[1] + 200,
                                        200, 150),
                            pygame.Rect(self.pos[0] + 350, self.pos[1] + 200,
                                        200, 150),
                            )
        self.init_items()
        self.gen_items("coconut_tree", 8)


class Island5(Island):
    def __init__(self):
        Island.__init__(self)
        self.init_from_file("map5")
        self.size = (6, 6)
        self.pos = (400, -600)
        self.collide_box = (pygame.Rect(self.pos[0], self.pos[1],
                                        self.size[0] * tile_size, self.size[1] * tile_size),)
        self.init_items()
        self.gen_items("rocks", 8)


class Island6(Island):
    def __init__(self):
        Island.__init__(self)
        self.init_from_file("map6")
        self.size = (16, 10)
        self.pos = (850, 550)
        self.collide_box = (pygame.Rect(self.pos[0] + 200, self.pos[1],
                                        400, 500),
                            pygame.Rect(self.pos[0], self.pos[1] + 100,
                                        800, 300),
                            )
        self.init_items()
        self.gen_items("tall_grass", 20)


class Island7(Island):
    def __init__(self):
        Island.__init__(self)
        self.init_from_file("map7")
        self.size = (16, 16)
        self.pos = (-1950, 1300)
        self.collide_box = (pygame.Rect(self.pos[0], self.pos[1],
                                        200, self.size[1] * tile_size),
                            pygame.Rect(self.pos[0], self.pos[1] + 600,
                                        800, 200),
                            )
        self.init_items()
        self.gen_items("sulfur_ore", 12)


class Island8(Island):
    def __init__(self):
        Island.__init__(self)
        self.init_from_file("map8")
        self.size = (17, 6)
        self.pos = (-300, 1600)
        self.collide_box = (pygame.Rect(self.pos[0], self.pos[1],
                                        self.size[0] * tile_size, self.size[1] * tile_size),)
        self.init_items()
        self.gen_items("tall_tree", 10)


def is_player_on_islands(islands, player):
    for island in islands:
        if island.is_player_on(player):
            return island
    return False


def is_platform_collide_islands(islands, platform):
    prect = pygame.Rect(platform.pos[0], platform.pos[1],
                        platform.size[0] * tile_size, platform.size[1] * tile_size)
    for island in islands:
        if island.collied_rect(prect):
            return True
    return False


def is_rect_collide_islands(islands, prect):
    for island in islands:
        if island.collied_rect(prect):
            return True
    return False
