from config import *
from island import *


class Item:
    def draw(self, pos):
        pass

    def update(self, pos):
        pass

    def is_dead(self):
        return True


class Tree(Item):
    def __init__(self):
        Item.__init__(self)
        self.tick = 0
        self.name = "tree"
        self.ready = False
        self.drop = (("item_wood", 5), ("item_sapling", random.randint(1, 2)))

    def update(self, pos=(0, 0)):
        if self.tick < 180:
            self.tick += 1

    def draw(self, pos=(0, 0)):
        if self.tick < 90:
            screen.blit(texture_lib["tree_stage1"], (pos[0], pos[1] - 50))
        elif self.tick < 180:
            screen.blit(texture_lib["tree_stage2"], (pos[0], pos[1] - 50))
        else:
            screen.blit(texture_lib["tree_stage3"], (pos[0], pos[1] - 50))
            if not self.ready:
                self.ready = True


class Melon(Item):
    def __init__(self):
        Item.__init__(self)
        self.name = "melon"
        self.tick = 0
        self.ready = False
        self.drop = (("item_melon", 1),)

    def update(self, pos=(0, 0)):
        if self.tick < 140:
            self.tick += 1

    def draw(self, pos=(0, 0)):
        if self.tick < 70:
            screen.blit(texture_lib["melon_stage1"], (pos[0], pos[1] - 50))
        elif self.tick < 140:
            screen.blit(texture_lib["melon_stage2"], (pos[0], pos[1] - 50))
        else:
            screen.blit(texture_lib["melon_stage3"], (pos[0], pos[1] - 50))
            if not self.ready:
                self.ready = True

class Anchor(Item):
    def __init__(self):
        Item.__init__(self)
        self.name = "anchor"
        self.direction = 0
        self.textures = ("stop", "right", "down", "left", "up")

    def draw(self, pos=(0, 0)):
        screen.blit(texture_lib[self.textures[self.direction]], pos)

    def next_dir(self):
        self.direction += 1
        if self.direction > 4:
            self.direction = 0

class Tile:
    def __init__(self, base="empty"):
        self.base = base
        self.float = 0
        self.float_cycle = ReCycle(4, 8, random.randint(0, 6))
        self.on_top = "empty"
        self.item = None

    def get_float(self):
        return self.float_cycle.current

    def is_tiled(self):
        return not self.base == "empty"

    def draw(self, pos):
        self.float = self.float_cycle.get()
        if self.base == "plank":
            screen.blit(texture_lib["plank"], (pos[0], pos[1] + self.float - 2))
        elif self.base == "empty":
            screen.blit(texture_lib["empty"], (pos[0], pos[1] + self.float - 2))
        if self.on_top != "empty":
            screen.blit(texture_lib[self.on_top], (pos[0], pos[1] + self.float - 2))

    def draw_item(self, pos):
        if self.item is not None:
            self.item.draw((pos[0], pos[1] + self.float - 2))
            self.update_item((pos[0], pos[1] + self.float - 2))

    def update_item(self, pos):
        if self.item is not None:
            self.item.update(pos)


class Platform:
    def __init__(self, game):
        self.size = [3, 3]
        self.area = [[Tile("plank"), Tile("plank"), Tile("plank")],
                     [Tile("plank"), Tile("plank"), Tile("plank")],
                     [Tile("plank"), Tile("empty"), Tile("plank")]]
        self.pos = [0, 0]
        self.animations = []
        self.area[0][0].on_top = "stone_base"
        self.area[0][0].item = Anchor()
        self.anchor = self.area[0][0].item
        self.game = game

    def move(self, x, y):
        self.pos[0] += x
        self.pos[1] += y

    def attempt_put(self, item, offset):
        tx, ty = self.hovered_tile((env_vars["mouse_x"] - offset[0], env_vars["mouse_y"] - offset[1]))
        if self.is_on_platform((tx, ty)):
            screen.blit(texture_lib[item], (tx * tile_size + self.pos[0] + offset[0] + 10,
                                            ty * tile_size + self.pos[1] + offset[1] + 10))
            if self.area[ty][tx].on_top == "soil" and self.area[ty][tx].item is None:
                pygame.draw.rect(screen, (3, 252, 132),
                                 pygame.Rect(tx * tile_size + self.pos[0] + offset[0],
                                             ty * tile_size + self.pos[1] + offset[1],
                                             tile_size, tile_size), 3)
                if env_vars["mouse_button_changed"] and env_vars["mouse_button_down"]:
                    if item == "item_sapling":
                        self.area[ty][tx].item = Tree()
                        ses[4].play_once()
                    elif item == "item_melon_seed":
                        self.area[ty][tx].item = Melon()
                        ses[4].play_once()
                    return True
                return False
            pygame.draw.rect(screen, (242, 55, 27),
                             pygame.Rect(tx * tile_size + self.pos[0] + offset[0],
                                         ty * tile_size + self.pos[1] + offset[1],
                                         tile_size, tile_size), 3)
        return False

    def attempt_place(self, item, offset):
        tx, ty = self.hovered_tile((env_vars["mouse_x"] - offset[0], env_vars["mouse_y"] - offset[1]))
        if self.is_on_platform((tx, ty)):
            screen.blit(texture_lib[item], (tx * tile_size + self.pos[0] + offset[0] + 10,
                                            ty * tile_size + self.pos[1] + offset[1] + 10))
            if not self.game.player.get_rect().colliderect(pygame.Rect(tx*tile_size+self.pos[0], ty*tile_size+self.pos[1],
                                                                       tile_size, tile_size)):
                if self.area[ty][tx].on_top == "empty" and self.area[ty][tx].item is None:
                    pygame.draw.rect(screen, (3, 252, 132),
                                     pygame.Rect(tx * tile_size + self.pos[0] + offset[0],
                                                 ty * tile_size + self.pos[1] + offset[1],
                                                 tile_size, tile_size), 3)
                    if env_vars["mouse_button_changed"] and env_vars["mouse_button_down"]:
                        self.area[ty][tx].on_top = item[5:]
                        ses[0].play_once()
                        return True
                    return False
            pygame.draw.rect(screen, (242, 55, 27),
                             pygame.Rect(tx * tile_size + self.pos[0] + offset[0],
                                         ty * tile_size + self.pos[1] + offset[1],
                                         tile_size, tile_size), 3)
        return False

    def attempt_harvest(self, offset):
        tx, ty = self.hovered_tile((env_vars["mouse_x"] - offset[0], env_vars["mouse_y"] - offset[1]))
        if self.is_on_platform((tx, ty)):
            if self.area[ty][tx].item is not None:
                if self.area[ty][tx].item.name == "anchor":
                    screen.blit(texture_lib["grab"], (tx * tile_size + self.pos[0] + offset[0] + 10,
                                                      ty * tile_size + self.pos[1] + offset[1] - 10))
                    if env_vars["mouse_button_changed"] and env_vars["mouse_button_down"]:
                        self.area[ty][tx].item.next_dir()
                        return False
                elif self.area[ty][tx].item.ready:
                    screen.blit(texture_lib["grab"], (tx * tile_size + self.pos[0] + offset[0] + 10,
                                                      ty * tile_size + self.pos[1] + offset[1] - 10))
                    if env_vars["mouse_button_changed"] and env_vars["mouse_button_down"]:
                        the_drop = self.area[ty][tx].item.drop
                        ses[4].play_once()
                        self.area[ty][tx].item = None
                        return the_drop
        return False

    def build(self, offset):
        if self.game.inventory.has("item_wood", 1):
            tx, ty = self.hovered_tile((env_vars["mouse_x"] - offset[0], env_vars["mouse_y"] - offset[1]))
            screen.blit(texture_lib["plank"], (tx * tile_size + self.pos[0] + offset[0],
                                               ty * tile_size + self.pos[1] + offset[1]))
            if self.is_valid_expansion((tx, ty)):
                pygame.draw.rect(screen, (3, 252, 132),
                                 pygame.Rect(tx * tile_size + self.pos[0] + offset[0],
                                             ty * tile_size + self.pos[1] + offset[1],
                                             tile_size, tile_size), 3)
                if env_vars["mouse_button_changed"] and env_vars["mouse_button_down"]:
                    self.game.inventory.take_item("item_wood", 1)
                    self.expand(tx, ty, "plank")
                    ses[0].play_once()
            else:
                pygame.draw.rect(screen, (242, 55, 27),
                                 pygame.Rect(tx * tile_size + self.pos[0] + offset[0],
                                             ty * tile_size + self.pos[1] + offset[1],
                                             tile_size, tile_size), 3)

    def hovered_tile(self, point):
        tx, ty = point[0] - self.pos[0], point[1] - self.pos[1]
        return int(tx//tile_size), int(ty//tile_size)

    def is_on_platform(self, point):
        if self.size[0] > point[0] >= 0 and self.size[1] > point[1] >= 0:
            return not self.area[point[1]][point[0]].base == "empty"
        return False

    def is_in_bound(self, point):
        if self.size[0] > point[0] >= 0 and self.size[1] > point[1] >= 0:
            return not self.area[point[1]][point[0]].base == "empty" and self.area[point[1]][point[0]].on_top == "empty"
        return False

    def is_valid_expansion(self, point):
        if self.size[0] > point[0] >= 0 and self.size[1] > point[1] >= 0:
            if self.area[point[1]][point[0]].base == "empty":
                return True
        if point[0] == -1 and self.size[1] > point[1] > -1:
            return not is_rect_collide_islands(self.game.islands, pygame.Rect(self.pos[0]-tile_size, self.pos[1],
                                                                              self.size[0] * tile_size + tile_size,
                                                                              self.size[1] * tile_size))
        if point[0] == self.size[0] and self.size[1] > point[1] > -1:
            return not is_rect_collide_islands(self.game.islands, pygame.Rect(self.pos[0], self.pos[1],
                                                                              self.size[0] * tile_size + tile_size,
                                                                              self.size[1] * tile_size))
        if point[1] == -1 and self.size[0] > point[0] > -1:
            return not is_rect_collide_islands(self.game.islands, pygame.Rect(self.pos[0], self.pos[1]-tile_size,
                                                                              self.size[0] * tile_size,
                                                                              self.size[1] * tile_size + tile_size))
        if point[1] == self.size[1] and self.size[0] > point[0] > -1:
            return not is_rect_collide_islands(self.game.islands, pygame.Rect(self.pos[0], self.pos[1],
                                                                              self.size[0] * tile_size,
                                                                              self.size[1] * tile_size + tile_size))
        return False

    def draw(self, offset=(200, 200)):
        for y in range(self.size[1]):
            for x in range(self.size[0]):
                self.area[y][x].draw((x*tile_size+offset[0]+self.pos[0], y*tile_size+offset[1]+self.pos[1]))
        for i in range(len(self.animations)-1, -1, -1):
            self.animations[i].draw(offset)
            if self.animations[i].is_dead():
                del self.animations[i]

    def new_row(self, head):
        if head:
            self.area.insert(0, [Tile() for _ in range(self.size[0])])
            self.size[1] += 1
            self.pos[1] -= tile_size
        else:
            self.area.append([Tile() for _ in range(self.size[0])])
            self.size[1] += 1

    def new_column(self, head):

        if head:
            for row in self.area:
                row.insert(0, Tile())
            self.size[0] += 1
            self.pos[0] -= tile_size
        else:
            for row in self.area:
                row.append(Tile())
            self.size[0] += 1

    def change_tile(self, x, y, base):
        self.area[y][x].base = base

    def expand(self, x, y, base="plank"):
        if x == -1 and y == -1:
            self.new_column(True)
            self.new_row(True)
            self.change_tile(0, 0, base)
        elif x == self.size[0] and y == self.size[1]:
            self.new_column(False)
            self.new_row(False)
            self.change_tile(x, y, base)
        elif x == -1 and y == self.size[1]:
            self.new_column(True)
            self.new_row(False)
            self.change_tile(0, y, base)
        elif x == self.size[0] and y == -1:
            self.new_column(False)
            self.new_row(True)
            self.change_tile(x, 0, base)
        elif x == -1:
            self.new_column(True)
            self.change_tile(0, y, base)
        elif y == -1:
            self.new_row(True)
            self.change_tile(x, 0, base)
        elif x == self.size[0]:
            self.new_column(False)
            self.change_tile(x, y, base)
        elif y == self.size[1]:
            self.new_row(False)
            self.change_tile(x, y, base)
        else:
            self.change_tile(x, y, base)

    def draw_items_back(self, player_y, offset=(200, 200)):
        for y in range(self.size[1]):
            for x in range(self.size[0]):
                if y*tile_size+self.pos[1] > player_y:
                    return
                self.area[y][x].draw_item((x*tile_size+offset[0]+self.pos[0], y*tile_size+offset[1]+self.pos[1]))

    def draw_items_fore(self, player_y, offset=(200, 200)):
        for y in range(self.size[1]):
            for x in range(self.size[0]):
                if y*tile_size+self.pos[1] > player_y:
                    self.area[y][x].draw_item((x*tile_size+offset[0]+self.pos[0], y*tile_size+offset[1]+self.pos[1]))
