from island import *
from animation import *
from platforms import *

class Player:
    def __init__(self, platform, islands, inventory):
        self.pos = [0, 0]
        self.draw_pos = default_center
        self.speed = 4
        self.speed_di = self.speed * 0.7
        self.dir = [0, 0]
        self.texture = [texture_lib["player_stand"], texture_lib["player_stand_left"],
                        texture_lib["player_walk"], texture_lib["player_walk_left"]]
        self.status = 0

        self.walk_cycle = Cycle(8, 3)
        self.stand_cycle = Cycle(4, 5)

        self.platform = platform
        self.islands = islands
        self.float = 0
        self.inventory = inventory

        self.is_on = platform
        self.crunch = None

    def get_holding_item(self):
        return self.inventory.pack[self.inventory.selected]

    def is_item_plantable(self):
        return self.inventory.pack[self.inventory.selected].item in plantable

    def is_item_placable(self):
        return self.inventory.pack[self.inventory.selected].item in placable

    def get_feet_rect(self):
        return (self.pos[0] + 19, self.pos[1] + 63),\
               (self.pos[0] + 56, self.pos[1] + 63),\
               (self.pos[0] + 19, self.pos[1] + 78),\
               (self.pos[0] + 56, self.pos[1] + 78)

    def get_rect(self):
        return pygame.Rect((self.pos[0] + 19, self.pos[1] + 63), (37, 15))

    def get_feet_center(self):
        return self.pos[0] + 37, self.pos[1] + 70

    def change_speed(self, new_speed):
        self.speed = new_speed
        self.speed_di = self.speed * 0.7

    def update(self):
        self.update_dateinfo()
        self.update_renderinfo()

    def update_dateinfo(self):
        k = pygame.key.get_pressed()
        if not (k[pygame.K_w] or k[pygame.K_a] or k[pygame.K_s] or k[pygame.K_d]):
            self.dir[0] = 0
            self.dir[1] = 0
        if self.dir[0] != 0 or self.dir[1] != 0:
            pre = self.pos[0], self.pos[1]
            if self.dir[0] == 0 or self.dir[1] == 0:
                self.pos[0] += self.dir[0] * self.speed
                self.pos[1] += self.dir[1] * self.speed
            else:
                self.pos[0] += self.dir[0] * self.speed_di
                self.pos[1] += self.dir[1] * self.speed_di

            pon = self.is_on
            self.is_on = self.platform
            for point in self.get_feet_rect():
                tx, ty = self.platform.hovered_tile(point)
                island = is_player_on_islands(self.islands, point)
                if island:
                    self.is_on = island
                elif not self.platform.is_in_bound((tx, ty)):
                    self.pos[0], self.pos[1] = pre[0], pre[1]
                    self.is_on = pon
                    break

    def update_renderinfo(self):
        if self.dir[0] == -1:
            self.status = 3
        elif self.dir[0] == 1:
            self.status = 2
        elif self.dir[1] == 0:
            if self.status == 2:
                self.status = 0
            elif self.status == 3:
                self.status = 1
        elif self.dir[1] != 0:
            if self.status == 0:
                self.status = 2
            elif self.status == 1:
                self.status = 3

        tx, ty = self.platform.hovered_tile(self.get_feet_center())
        if type(self.is_on) is Platform:
            self.float = self.platform.area[ty][tx].get_float()
        else:
            self.float = 0
        self.draw_pos = (default_center[0], default_center[1]+self.float)

    def draw(self):
        if self.status == 2 or self.status == 3:
            temp = self.walk_cycle.get()
            screen.blit(self.texture[self.status], self.draw_pos,
                        pygame.Rect(0, temp * player_size, player_size, player_size))
            if self.walk_cycle.changed() and (temp == 3 or temp == 7):
                self.platform.animations.append(WalkDust(self.pos, self.status == 2))
            if self.walk_cycle.changed() and (temp == 1 or temp == 5):
                ses[2].play_once()
            if self.inventory.pack[self.inventory.selected].item != "empty":
                tp = hand_pos[self.status][temp]
                screen.blit(texture_lib[self.inventory.pack[self.inventory.selected].item],
                            (tp[0] - 15 + self.draw_pos[0], tp[1] - 15 + self.draw_pos[1]))
        elif self.status == 0 or self.status == 1:
            screen.blit(self.texture[self.status], self.draw_pos,
                        pygame.Rect(0, self.stand_cycle.get() * player_size, player_size, player_size))
            if self.inventory.pack[self.inventory.selected].item != "empty":
                tp = hand_pos[self.status][self.stand_cycle.current]
                screen.blit(texture_lib[self.inventory.pack[self.inventory.selected].item],
                            (tp[0] - 15 + self.draw_pos[0], tp[1] - 15 + self.draw_pos[1]))
        if self.crunch is not None:
            self.crunch.draw()
            if self.crunch.is_dead():
                self.crunch = None


    def draw_shadow(self):
        pygame.draw.ellipse(screen, (100, 100, 100), pygame.Rect((self.draw_pos[0] + 19, self.draw_pos[1] + 63), (37, 15)))

    def handle_event(self, event):
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_w:
                self.dir[1] -= 1
            elif event.key == pygame.K_s:
                self.dir[1] += 1
            elif event.key == pygame.K_a:
                self.dir[0] -= 1
            elif event.key == pygame.K_d:
                self.dir[0] += 1

        elif event.type == pygame.KEYUP:
            if event.key == pygame.K_w:
                self.dir[1] += 1
            elif event.key == pygame.K_s:
                self.dir[1] -= 1
            elif event.key == pygame.K_a:
                self.dir[0] += 1
            elif event.key == pygame.K_d:
                self.dir[0] -= 1


class Slot:
    def __init__(self):
        self.item = "empty"
        self.count = 0

    def put(self, stack):
        self.item = stack[0]
        self.count = stack[1]

    def take(self, num):
        self.count -= num
        if self.count == 0:
            self.item = "empty"

    def switch(self, slot):
        if slot.item != self.item:
            self.item, slot.item = slot.item, self.item
            self.count, slot.count = slot.count, self.count
        else:
            self.count += slot.count
            slot.count = 0
            slot.item = "empty"

    def draw(self, pos):
        screen.blit(texture_lib[self.item], pos)
        num = item_font.render(str(self.count), False, (0, 0, 0))
        screen.blit(num, (pos[0] + 5, pos[1] - 10))

    def is_hovered(self, pos):
        return pygame.Rect(pos, (30, 30)).collidepoint(env_vars["mouse_x"], env_vars["mouse_y"])


class Inventory:
    def __init__(self):
        self.pack = [Slot() for _ in range(20)]
        self.hand = Slot()
        self.pack[0].put(("item_sapling", 1))
        self.pack[1].put(("item_melon", 1))
        self.pack[2].put(("item_soil", 2))
        self.selected = 0

    def has(self, item, count):
        for slot in self.pack:
            if slot.item == item:
                if slot.count >= count:
                    return True
        return False

    def put_item(self, item, count):
        for slot in self.pack:
            if slot.item == item:
                slot.count += count
                return True
        for slot in self.pack:
            if slot.item == "empty":
                slot.item = item
                slot.count = count
                return True
        return False

    def take_item(self, item, count):
        for slot in self.pack:
            if slot.item == item:
                if slot.count < count:
                    return False
                slot.count -= count
                if slot.count == 0:
                    slot.item = "empty"
                return True
        return False

    def draw_pack(self):
        screen.blit(texture_lib["pack"], (150, 50))
        screen.blit(texture_lib["inventory"], (150, 500))
        for i in range(5):
            if self.pack[i].is_hovered((178 + 53 * i, 512)):
                screen.blit(texture_lib["select_slot"], (178 + 53 * i, 512))
                if env_vars["mouse_button_down"] and env_vars["mouse_button_changed"]:
                    self.pack[i].switch(self.hand)
            if self.pack[i].item != "empty":
                self.pack[i].draw((178 + 53 * i, 512))
        for i in range(3):
            for ii in range(5):
                if self.pack[i * 5 + ii + 5].is_hovered((178 + 53 * ii, 60 + i * 50)):
                    screen.blit(texture_lib["select_slot"], (178 + 53 * ii, 60 + i * 50))
                    if env_vars["mouse_button_down"] and env_vars["mouse_button_changed"]:
                        self.pack[i * 5 + ii + 5].switch(self.hand)
                if self.pack[i * 5 + ii + 5].item != "empty":
                    self.pack[i * 5 + ii + 5].draw((178 + 53 * ii, 60 + i * 50))
        if self.hand.item != "empty":
            self.hand.draw((env_vars["mouse_x"], env_vars["mouse_y"]))

    def draw_inventory(self):
        screen.blit(texture_lib["inventory"], (150, 500))
        screen.blit(texture_lib["selected_slot"], (self.selected * 53 + 179, 510))
        for i in range(5):
            if self.pack[i].item != "empty":
                self.pack[i].draw((178 + 53 * i, 512))

    def handle_event(self, event):
        if event.type == pygame.KEYUP:
            if event.key == pygame.K_RIGHT:
                self.selected += 1
                if self.selected > 4:
                    self.selected = 0
            elif event.key == pygame.K_LEFT:
                self.selected -= 1
                if self.selected < 0:
                    self.selected = 4
        elif event.type == pygame.MOUSEBUTTONDOWN:
            if event.button == 4:
                self.selected += 1
                if self.selected > 4:
                    self.selected = 0
            elif event.button == 5:
                self.selected -= 1
                if self.selected < 0:
                    self.selected = 4


class Crafting:
    def __init__(self, inventory, craft):
        self.inventory = inventory
        self.selected = 0
        self.craft = craft
        self.crafts = len(craft)
        self.can_craft = False
        self.chosen = "workbench"

    def draw(self):
        current = 0
        for key in self.craft:
            pos = (400, 280 + current * 50 - self.selected * 50)
            if current == self.selected:
                self.chosen = key
                self.can_craft = True
                for i, item in enumerate(self.craft[key]):
                    screen.blit(texture_lib[item[0]], (pos[0] + 65 + 35 * i, pos[1] + 5))
                    if self.inventory.has(item[0], item[1]):
                        screen.blit(item_font.render(str(item[1]), False, (30, 30, 30)),
                                    (pos[0] + 75 + 35 * i, pos[1] + 25))
                    else:
                        screen.blit(item_font.render(str(item[1]), False, (255, 30, 30)),
                                    (pos[0] + 75 + 35 * i, pos[1] + 25))
                        self.can_craft = False
                if self.can_craft:
                    screen.blit(texture_lib["craft_result"], pos)
                else:
                    screen.blit(texture_lib["craft_unable"], pos)
                if key[1] != 1:
                    screen.blit(item_font.render(str(key[1]), False, (30, 30, 30)), (pos[0] + 15, pos [1] + 30))
            screen.blit(texture_lib[key[0]], (pos[0] + 5, pos[1] + 5))
            current += 1

    def handle_events(self, event):
        if event.type == pygame.MOUSEBUTTONDOWN:
            if event.button == 5:
                self.selected += 1
                if self.selected >= self.crafts:
                    self.selected = 0
            elif event.button == 4:
                self.selected -= 1
                if self.selected < 0:
                    self.selected = self.crafts - 1
        elif event.type == pygame.KEYUP:
            if event.key == pygame.K_RETURN:
                if self.can_craft:
                    for item in self.craft[self.chosen]:
                        self.inventory.take_item(item[0], item[1])
                    self.inventory.put_item(self.chosen[0], self.chosen[1])
                    return True
            elif event.key == pygame.K_UP or event.key == pygame.K_w:
                self.selected -= 1
                if self.selected < 0:
                    self.selected = self.crafts - 1
            elif event.key == pygame.K_DOWN or event.key == pygame.K_s:
                self.selected += 1
                if self.selected >= self.crafts:
                    self.selected = 0
        return False


