from game import *

class BlenderOpen:
    def __init__(self):
        self.num = 30
        self.lines = [(600/self.num+i) for i in range(self.num)]

    def is_dead(self):
        return self.lines[-1] == 0

    def draw(self):
        for i in range(self.num):
            if self.lines[i] > 0:
                if self.lines[i] > 20:
                    screen.fill(0, pygame.Rect(0, i*20, 600, 20))
                else:
                    screen.fill(0, pygame.Rect(0, i * 20 + self.lines[i]/2, 600, self.lines[i]))
                self.lines[i] -= 1


class BlenderClose:
    def __init__(self):
        self.num = 30
        self.lines = [-i for i in range(self.num)]

    def is_dead(self):
        return self.lines[-1] > 20

    def draw(self):
        for i in range(self.num):
            if self.lines[i] > 0:
                if self.lines[i] > 20:
                    screen.fill(0, pygame.Rect(0, i * 20, 600, 20))
                else:
                    screen.fill(0, pygame.Rect(0, i * 20 + 10 - self.lines[i] / 2, 600, self.lines[i]))
            self.lines[i] += 1

class StoryStart:
    def __init__(self):
        self.blender = BlenderOpen()
        self.ended = False
        self.current = -1
        self.transition = 0
        self.b = 0
        self.stories = ("start_scene1", "start_scene2", "start_scene3")

    def handle_event(self, event):
        if event.type == pygame.MOUSEBUTTONUP:
            if self.transition == -1 and not self.blender:
                if self.current == 2:
                    self.blender = BlenderClose()
                else:
                    self.transition = 20

    def draw(self):
        if self.blender:
            if self.b == 0:
                screen.blit(texture_lib[self.stories[0]], (0, 0))
            elif self.b == 1:
                screen.blit(texture_lib[self.stories[2]], (0, 0))
            self.blender.draw()
            if self.blender.is_dead():
                self.blender = False
                if self.b == 1:
                    self.ended = True
                self.b += 1
        elif self.transition > 0:
            self.transition -= 1
            screen.blit(texture_lib[self.stories[self.current]], (self.transition*30 - 600, 0))
            screen.blit(texture_lib[self.stories[self.current+1]], (self.transition*30, 0))
        elif self.transition == 0:
            self.current += 1
            self.transition = -1
            screen.blit(texture_lib[self.stories[self.current]], (0, 0))
        else:
            screen.blit(texture_lib[self.stories[self.current]], (0, 0))


class StoryEnd:
    def __init__(self):
        self.blender = BlenderOpen()
        self.ended = False
        self.current = -1
        self.transition = 0
        self.b = 0
        self.stories = ("end_scene1", "end_scene2", "end_scene3")

    def handle_event(self, event):
        if event.type == pygame.MOUSEBUTTONUP:
            if self.transition == -1 and not self.blender:
                if self.current == 2:
                    self.blender = BlenderClose()
                else:
                    self.transition = 20

    def draw(self):
        if self.blender:
            if self.b == 0:
                screen.blit(texture_lib[self.stories[0]], (0, 0))
            elif self.b == 1:
                screen.blit(texture_lib[self.stories[2]], (0, 0))
            self.blender.draw()
            if self.blender.is_dead():
                self.blender = False
                if self.b == 1:
                    self.ended = True
                self.b += 1
        elif self.transition > 0:
            self.transition -= 1
            screen.blit(texture_lib[self.stories[self.current]], (self.transition*30 - 600, 0))
            screen.blit(texture_lib[self.stories[self.current+1]], (self.transition*30, 0))
        elif self.transition == 0:
            self.current += 1
            self.transition = -1
            screen.blit(texture_lib[self.stories[self.current]], (0, 0))
        else:
            screen.blit(texture_lib[self.stories[self.current]], (0, 0))


class Death:
    def __init__(self, name):
        self.blender = BlenderOpen()
        self.order = 0
        self.name = name
        self.dead = False

    def handle_event(self, event):
        if event.type == pygame.MOUSEBUTTONUP:
            if not self.blender:
                self.blender = BlenderClose()

    def draw(self):
        screen.blit(texture_lib[self.name], (0, 0))
        if self.blender:
            self.blender.draw()
            if self.blender.is_dead():
                self.blender = False
                if self.order == 1:
                    self.dead = True
                self.order += 1


class MainMenu:
    def __init__(self):
        self.wave_cycle = Cycle(200, 2)
        self.wave_ups = (ReCycle(30, 3, 9), ReCycle(30, 3, 19), ReCycle(30, 3, 29))
        self.wave_up_num = [0, 0, 0]
        self.ship_cycle = ReCycle(20, 2)
        self.castaway_cycle = ReCycle(20, 2, 10)
        self.cloud_cycle1 = ReCycle(13, 2)
        self.cloud_cycle2 = ReCycle(17, 2)
        self.blender = BlenderOpen()
        self.ended = -1

    def handle_event(self, event):
        if event.type == pygame.MOUSEBUTTONUP:
            if self.blender:
                self.ended += 1
            self.blender = BlenderClose()

    def draw(self):
        current = self.wave_cycle.get()
        for i in range(3):
            self.wave_up_num[i] = self.wave_ups[i].get()
        screen.fill((181, 202, 235))
        screen.blit(texture_lib["cloud"], (200, -80 - self.cloud_cycle1.get()))
        screen.blit(texture_lib["cloud"], (400, -60 - self.cloud_cycle2.get()))

        screen.blit(texture_lib["island"], (0, 130))

        screen.blit(texture_lib["wave_layer1"], (current * 3, 200 + self.wave_up_num[0]))
        screen.blit(texture_lib["wave_layer1"], (0, 200 + self.wave_up_num[0]),
                    pygame.Rect(600 - current * 3, 0, current * 3, 200))

        screen.blit(texture_lib["sunk_ship"], (330, 240 + self.castaway_cycle.get()))

        screen.blit(texture_lib["wave_layer2"], (-current * 3, 300 + self.wave_up_num[1]))
        screen.blit(texture_lib["wave_layer2"], (600-current * 3, 300 + self.wave_up_num[1]),
                    pygame.Rect(0, 0, current * 3, 200))

        screen.blit(texture_lib["floating_plank"], (130, 420 + self.ship_cycle.get()))

        screen.blit(texture_lib["wave_layer3"], (current * 3, 400 + self.wave_up_num[2]))
        screen.blit(texture_lib["wave_layer3"], (0, 400 + self.wave_up_num[2]),
                    pygame.Rect(600 - current * 3, 0, current * 3, 200))

        screen.blit(texture_lib["title"], (50, 50))

        if self.blender:
            self.blender.draw()
            if self.blender.is_dead():
                self.blender = False
                self.ended += 1

bgs[0].play_non_stop()
g = Game()
s1 = StoryStart()
s2 = StoryEnd()
s3 = Death("hunger_death")
s4 = Death("thirst_death")
m = MainMenu()
started = 0
game_blend = BlenderOpen()
game_count = 0
temp = 0
prev_stage = 0

while True:
    env_vars["mouse_button_changed"] = False
    for event in pygame.event.get():
        helped = False
        if event.type == pygame.QUIT:
            pygame.quit()
            sys.exit()
        elif event.type == pygame.MOUSEMOTION:
            new_mouse_pos = pygame.mouse.get_pos()
            env_vars["delta_x"] = new_mouse_pos[0] - env_vars["mouse_x"]
            env_vars["delta_y"] = new_mouse_pos[1] - env_vars["mouse_y"]
            env_vars["mouse_x"], env_vars["mouse_y"] = new_mouse_pos
        elif event.type == pygame.MOUSEBUTTONUP:
            env_vars["mouse_button_down"] = False
            env_vars["mouse_button_changed"] = True
            if env_vars["mouse_x"] > 525 and env_vars["mouse_y"] > 525:
                if started != 10:
                    prev_stage = started
                    started = 10
                else:
                    started = prev_stage
                helped = True
        elif event.type == pygame.MOUSEBUTTONDOWN:
            env_vars["mouse_button_down"] = True
            env_vars["mouse_button_changed"] = True
        if not helped:
            if started == 2:
                g.handle_event(event)
            elif started == 0:
                m.handle_event(event)
            elif started == 1:
                s1.handle_event(event)
            elif started == 3:
                s2.handle_event(event)
            elif started == 4:
                s3.handle_event(event)
            elif started == 5:
                s4.handle_event(event)
    env_vars["keys"] = pygame.key.get_pressed()
    if started == 2:
        g.update()
        if not game_blend and (g.mission.finished or g.hunger < 0 or g.thirst < 0):
            if g.mission.finished:
                temp = 3
            elif g.hunger < 0:
                temp = 4
            elif g.thirst < 0:
                temp = 5
            game_blend = BlenderClose()
        else:
            g.draw()
            if game_blend:
                game_blend.draw()
                if game_blend.is_dead():
                    game_blend = False
                    if game_count == 1:
                        if g.mission.finished:
                            started = temp
                        elif g.hunger < 0:
                            started = temp
                        elif g.thirst < 0:
                            started = temp
                    game_count += 1
    elif started == 0:
        m.draw()
        if m.ended == 1:
            started = 1
    elif started == 1:
        s1.draw()
        if s1.ended:
            started = 2
            bgs[1].play_non_stop()
    elif started == 3:
        s2.draw()
        if s2.ended:
            bgs[0].play_non_stop()
            started = 0
            g = Game()
            m = MainMenu()
            s1 = StoryStart()
            s2 = StoryEnd()
            s3 = Death("hunder_death")
            s4 = Death("thirst_death")
            game_blend = BlenderOpen()
            game_count = 0
            prev_stage = 0
    elif started == 4:
        s3.draw()
        if s3.dead:
            bgs[0].play_non_stop()
            started = 0
            g = Game()
            m = MainMenu()
            s1 = StoryStart()
            s2 = StoryEnd()
            s3 = Death("hunder_death")
            s4 = Death("thirst_death")
            game_blend = BlenderOpen()
            game_count = 0
            prev_stage = 0
    elif started == 5:
        s4.draw()
        if s4.dead:
            bgs[0].play_non_stop()
            started = 0
            g = Game()
            m = MainMenu()
            s1 = StoryStart()
            s2 = StoryEnd()
            s3 = Death("hunder_death")
            s4 = Death("thirst_death")
            game_blend = BlenderOpen()
            game_count = 0
            prev_stage = 0
    elif started == 10:
        screen.blit(texture_lib["help"], (0, 0))
    screen.blit(texture_lib["question"], (525, 525))

    pygame.display.flip()
    clock.tick(40)
